import numpy
import logging
import sys 
from apgl.util.Latex import Latex
from apgl.util.PathDefaults import PathDefaults

#Produce latex tables from the benchmark results
logging.basicConfig(stream=sys.stdout, level=logging.DEBUG)

sampleSizes = numpy.array([50, 100, 200])
foldsSet = numpy.arange(2, 11, 2)
outputDir = PathDefaults.getOutputDir() + "modelPenalisation/"

datasetNames = []
datasetNames.append("banana")
datasetNames.append("breast-cancer")
datasetNames.append("diabetis")
datasetNames.append("flare-solar")
datasetNames.append("german")
datasetNames.append("heart") 
datasetNames.append("image")
datasetNames.append("ringnorm")
datasetNames.append("splice")
datasetNames.append("thyroid")
datasetNames.append("titanic")
datasetNames.append("twonorm")
datasetNames.append("waveform")

numDatasets = len(datasetNames)

numMethods = 3 
overallErrors = numpy.zeros((numDatasets, sampleSizes.shape[0], foldsSet.shape[0], numMethods))
overallErrorTable = Latex.array1DToRow(foldsSet) + "\n"

for i in range(len(datasetNames)):
    print("="*50)
    print(datasetNames[i])
    print("="*50)
    outfileName = outputDir + datasetNames[i] + "Results.npz"
    data = numpy.load(outfileName)

    errors = data["arr_0"]
    params = data["arr_1"]

    rowNames = [""]
    for j in range(sampleSizes.shape[0]):
        rowNames.append("CrossV" + " n=" + str(sampleSizes[j]))
        rowNames.append("PenBIC" + " n=" + str(sampleSizes[j]))
        rowNames.append("PenVF" + " n=" + str(sampleSizes[j]))

    errorTable = Latex.array1DToRow(foldsSet) + "\n"
    paramsTableC = Latex.array1DToRow(foldsSet) + "\n"
    paramsTableGamma = Latex.array1DToRow(foldsSet) + "\n"

    for j in range(sampleSizes.shape[0]):
        meanErrors = numpy.mean(errors, 0)
        stdErrors = numpy.std(errors, 0)
        errorTable += Latex.array2DsToRows(meanErrors[j, :, :].T, stdErrors[j, :, :].T) + "\n"

        overallErrors[i, j, :, :] = meanErrors[j, :, :]
        
        meanParams = numpy.mean(params, 0)
        stdParams = numpy.std(params, 0)
        paramsTableC += Latex.array2DsToRows(meanParams[j, :, :, 0].T, stdParams[j, :, :, 0].T) + "\n"
        paramsTableGamma += Latex.array2DsToRows(meanParams[j, :, :, 1].T, stdParams[j, :, :, 1].T) + "\n"

    errorTable = Latex.addRowNames(rowNames, errorTable)
    paramsTableC = Latex.addRowNames(rowNames, paramsTableC)
    paramsTableGamma = Latex.addRowNames(rowNames, paramsTableGamma)

    print(errorTable)
    print(paramsTableC)
    print(paramsTableGamma)


print("="*50)
print("Overall")
print("="*50)

overallMeanErrors = numpy.mean(overallErrors, 0)
overallStdErrors = numpy.std(overallErrors, 0)

for j in range(sampleSizes.shape[0]):
    overallErrorTable += Latex.array2DsToRows(overallMeanErrors[j, :, :].T, overallStdErrors[j, :, :].T) + "\n"
overallErrorTable = Latex.addRowNames(rowNames, overallErrorTable)

print(overallErrorTable)
