import sys 
import numpy
import logging
import svmutil 
from apgl.predictors.LibSVM import LibSVM
from apgl.util.PathDefaults import PathDefaults
from apgl.util.Evaluator import Evaluator
from apgl.util.Util import Util
from apgl.data.Standardiser import Standardiser

"""
Let's run cross validation and model penalisation over the benchmark datasets.
"""

def dictToArray(dictList):
    """
    A bad function to convert a list of dicts in libsvm format to a numpy array.
    """
    numExamples = len(dictList)
    numFeatures = 2

    X = numpy.zeros((numExamples, numFeatures))

    for i in range(len(dictList)):
        val = dictList[i]
        for j in range(X.shape[1]):
            X[i, j] = val[j+1]

    return X 

logging.basicConfig(stream=sys.stdout, level=logging.DEBUG)

sampleSizes = numpy.array([50, 100, 200])
#sampleSizes = numpy.array([50])
foldsSet = numpy.arange(2, 11)
#foldsSet = numpy.array([5])
dataDir = PathDefaults.getDataDir() + "modelPenalisation/"
trainDatasetPre = dataDir + "mix2D_train."
testDatasetName = dataDir + "mix2D_train.test"

numMethods = 3
numRepetitions = 10
#numRepetitions = 1
numParams = 2
errors = numpy.zeros((numRepetitions, len(sampleSizes), foldsSet.shape[0], numMethods))
params = numpy.zeros((numRepetitions, len(sampleSizes), foldsSet.shape[0], numMethods, numParams))

testY, testX = svmutil.svm_read_problem(testDatasetName)
testX = dictToArray(testX)
testY = numpy.array(testY)

for j in range(numRepetitions):
    Util.printIteration(j, 1, numRepetitions, "Repetition: ")
    for k in range(sampleSizes.shape[0]):
        sampleSize = sampleSizes[k]
        trainDatasetName = trainDatasetPre + str(sampleSize) + "." + str(j+1)

        trainY, trainX = svmutil.svm_read_problem(trainDatasetName)
        trainX = dictToArray(trainX)
        trainY = numpy.array(trainY)
        
        preprocessor = Standardiser()
        trainX = preprocessor.scaleArray(trainX)
        testX = preprocessor.scaleArray(testX)

        for m in range(foldsSet.shape[0]):
            folds = foldsSet[m]
            logging.info("Sample size " + str(sampleSize) + ", number of folds " + str(folds))
            
            perm = numpy.random.permutation(trainX.shape[0])
            trainInds = perm[0:sampleSize]
            validX = trainX[trainInds, :]
            validY = trainY[trainInds]
            svm = LibSVM()

            #Cross validation
            #bestSVM = svm.vfcvRbf(validX, validY, folds)
            logging.debug("Running V-fold cross validation")
            bestSVM = svm.parallelVfcvRbf(validX, validY, folds)
            predY = bestSVM.predict(testX)
            errors[j, k, m, 0] = Evaluator.binaryError(testY, predY)
            params[j, k, m, 0, :] = numpy.array([bestSVM.getC(), bestSVM.getKernelParams()])

            #BIC penalisation
            logging.debug("Running BIC penalisation")
            Cv = float((folds-1) * numpy.log(validX.shape[0])/2)
            bestSVM = svm.parallelVfPenRbf(validX, validY, folds, Cv)
            predY = bestSVM.predict(testX)
            errors[j, k, m, 1] = Evaluator.binaryError(testY, predY)
            params[j, k, m, 1, :] = numpy.array([bestSVM.getC(), bestSVM.getKernelParams()])

            #v fold penalisation
            logging.debug("Running V-fold penalisation")
            Cv = float(folds-1)
            bestSVM = svm.parallelVfPenRbf(validX, validY, folds, Cv)
            predY = bestSVM.predict(testX)
            params[j, k, m, 2, :] = numpy.array([bestSVM.getC(), bestSVM.getKernelParams()])

meanErrors = numpy.mean(errors, 0)
print(meanErrors)

meanParams = numpy.mean(params, 0)
print(meanParams)

outputDir = PathDefaults.getOutputDir() + "modelPenalisation/"
outfileName = outputDir + "toyResults"
numpy.savez(outfileName, errors, params)
logging.debug("Saved results as file " + outfileName + ".npz")

        